unit d5Utils;

interface

uses classes;

type

{ TObjectList class }

  TObjectList = class
  private
    FList : TList;
    FOwnsObjects: Boolean;
    procedure SetCount(Value : integer);
    function GetCount : integer;
  protected
    function GetItem(Index: Integer): TObject;
    procedure SetItem(Index: Integer; AObject: TObject);
  public
    constructor Create(AOwnsObjects: Boolean);

    function Add(AObject: TObject): Integer;
    function Remove(AObject: TObject): Integer;
    procedure Delete(Index : integer);
    function IndexOf(AObject: TObject): Integer;
    procedure Insert(Index: Integer; AObject: TObject);
    procedure Clear;

    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
    property Items[Index: Integer]: TObject read GetItem write SetItem; default;
    property Count : integer read GetCount write SetCount;

    property List : TList read FList;
  end;

  procedure FreeAndNil(var Obj);


implementation

procedure FreeAndNil(var Obj);
var
  P: TObject;
begin
  P := TObject(Obj);
  TObject(Obj) := nil;
  P.Free;
end;

{ TObjectList }

function TObjectList.Add(AObject: TObject): Integer;
begin
  Result := FList.Add(AObject);
end;

constructor TObjectList.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FList := TList.Create;
  FOwnsObjects := AOwnsObjects;
end;

function TObjectList.GetItem(Index: Integer): TObject;
begin
  Result := FList.Items[Index];
end;

function TObjectList.IndexOf(AObject: TObject): Integer;
begin
  Result := FList.IndexOf(AObject);
end;

procedure TObjectList.Insert(Index: Integer; AObject: TObject);
begin
  FList.Insert(Index, AObject);
end;

function TObjectList.Remove(AObject: TObject): Integer;
begin
  Result := IndexOf(AObject);
  if Result < 0 then exit;
  Delete(Result);
end;

procedure TObjectList.SetItem(Index: Integer; AObject: TObject);
begin
  FList.Items[Index] := AObject;
end;

procedure TObjectList.Delete(Index : integer);
begin
  if OwnsObjects then Items[Index].Free;
  FList.Delete(Index);
end;

procedure TObjectList.Clear;
var i : integer;
begin
  if OwnsObjects then
    for i := 0 to Count - 1 do Items[i].Free;
  FList.Clear;
end;

procedure TObjectList.SetCount(Value : integer);
begin
  FList.Count := Value;
end;

function TObjectList.GetCount : integer;
begin
  Result := FList.Count;
end;

end.
